angular.module('starter.controllers', [])
.controller('AppCtrl', function($scope, $rootScope, userService, $state, $ionicHistory, purchaseService) {
	$rootScope.$on('app.userChanged', function() {
		$scope.userInfo = userService.getUserInfo();
		if ($scope.userInfo) {
			purchaseService.syncTickets();
		}
	});
	$rootScope.$emit('app.userChanged');
	$scope.login = function() {
		userService.showLogin();
	};
	$scope.open = function(state) {
		if (userService.isLoggedIn()) {
			$ionicHistory.nextViewOptions({
	          disableBack: true
	        });
			$state.go(state);
		} else {
			userService.showLogin(function() {
				$ionicHistory.nextViewOptions({
		          disableBack: true
		        });
				$state.go(state);
			});
		}
	};
	$scope.isActive = function(state) {
		return $state.current.name == state;
	};
})
.controller('EventsCtrl', function($scope, $ionicModal, $rootScope, eventService, _, userService, $rootScope, storage, messages, $filter, notifications, notificationService) {
	$scope.search = {query: '', state: "all"};
	$scope.lastLoaded = false;
	$scope.loading = false;
	$scope.events = [];
	$scope.$watch('search.query', function() {
		$scope.doRefresh();
	});
	$scope.$watch('search.state', function() {
		$scope.doRefresh();
	});
	$rootScope.$on('app.userChanged', function() {
		$scope.doRefresh();
	});
	$scope.doRefresh = function() {
		eventService.countNotViewed(function(notViewed) {
			$rootScope.notViewedEvents = notViewed;
		});
		notificationService.updateBadge();
		$scope.events = [];
		$scope.lastLoaded = false;
		$scope.loadMore();
	};
	$scope.loadMore = function() {
		if(storage.getObject('pendingPurchase') == null) {
			if (!$scope.loading) {
				$scope.loading = true;
				eventService.listByCompany(companyId, $scope.events.length, $scope.search.query, $scope.search.state, function(result) {
					if (!result || result.length == 0) {
						$scope.lastLoaded = true;
					} else {
						$scope.events = $scope.events.concat(result);
					}
				    $scope.$broadcast('scroll.infiniteScrollComplete');
				    $scope.$broadcast('scroll.refreshComplete');
				    $scope.loading = false;
			    });
			}
		}
	};
	$scope.$on('$stateChangeSuccess', function (event, toState, toParams, fromState, fromParams) {
		if (toState.name == "app.events") {
			$scope.events = [];
			$scope.doRefresh();
		}
	});
	$scope.openEvent = function(event) {
		if(!event.disableSales) {
			eventService.openEvent(event, $scope);
		} else {
			messages.info($filter('translate')('views.events.coming-soon'));
		}
	};
	$scope.toggleLike = function(event) {
		userService.showLogin(function() {
			eventService.toggleLike(event, function(liked){
				event.liked = liked;
			});
		});
	};
	$rootScope.$on('events.liked', function(event, args) {
		_.findWhere($scope.events, {id: args.eventId}).liked = args.liked;
	});
	$scope.favorites = function() {
		if (userService.isLoggedIn()) {
			$scope.search.state = 'liked';
		} else {
			userService.showLogin(function() {
				$scope.search.state = 'liked';
			});
		}
	};
	$scope.markViewed = function(event) {
		if (!event.viewed) {
			event.viewed = true;
			eventService.view(event, function(notViewed) {
				$rootScope.notViewedEvents = notViewed;
			});
		}
	};
})
.controller('NotificationsCtrl', function($scope, notificationService, $rootScope) {
    $scope.lastLoaded = false;
    $scope.loading = false;
    $scope.items = [];
    
    $scope.doRefresh = function() {
        notificationService.countNotViewed(function(notViewed) {
            $rootScope.notViewedNotifications = notViewed;
        });
        $scope.items = [];
        $scope.lastLoaded = false;
        $scope.loadMore();
    };
    $scope.loadMore = function() {
        if (!$scope.loading) {
            $scope.loading = true;
            notificationService.list($scope.items.length, function(result) {
                if (!result || result.length == 0) {
                    $scope.lastLoaded = true;
                } else {
                    $scope.items = $scope.items.concat(result);
                }
                $scope.$broadcast('scroll.infiniteScrollComplete');
                $scope.$broadcast('scroll.refreshComplete');
                $scope.loading = false;
            });
        }
    };
    $scope.$on('$stateChangeSuccess', function (event, toState, toParams, fromState, fromParams) {
        if (toState.name == "app.notifications") {
            $scope.doRefresh();
        }
    });
    $scope.markViewed = function(notification) {
        if (!notification.viewed) {
            notification.viewed = true;
            notificationService.view(notification, function(notViewed) {
                $rootScope.notViewedNotifications = notViewed;
            });
        }
    };
})
.controller('SettingsCtrl', function($scope, $rootScope, $state, $ionicHistory, userService) {
	$scope.myProfile = function() {
		$state.go('app.myProfile');
	};
	$scope.changePassword = function() {
		$state.go('app.changePassword');
    };
	$scope.logout = function() {
		userService.logout(function() {
			$ionicHistory.nextViewOptions({
	          disableAnimate: true,
	          disableBack: true
	        });
			$state.go("app.events");
		});
	};
})
.controller('MyTicketsCtrl', function($scope, $rootScope, backend, purchaseService, $ionicModal, $filter, $ionicSlideBoxDelegate) {
	$rootScope.tickets = [];
	$scope.$on('$stateChangeSuccess', function (event, toState, toParams, fromState, fromParams) {
	    if (toState.name == "app.myTickets") {
	    	purchaseService.syncTickets();
	    	$ionicSlideBoxDelegate.update();
	    }
	});
	$rootScope.$watch('tickets', function() {
		$ionicSlideBoxDelegate.update();
	}, true);
	$scope.showTerms = function() {
		$scope.modalUrl = $filter("translate")("views.buyTicket.termsUrl");
		$scope.modalTitle = $filter("translate")("views.buyTicket.termsTitle");
		$ionicModal.fromTemplateUrl('templates/externalPage.html', {
	      scope: $scope,
	      animation: 'slide-in-up'
	    }).then(function(modal) {
	    	$scope.externalPageModal = modal;
		    $scope.externalPageModal.show();
	    });
		$scope.$on('$destroy', function() {
			$scope.externalPageModal.remove();
		});
	};
})
.controller('ChangePasswordCtrl', function($scope, userService, messages, $translate, $ionicHistory) {
	$scope.form = {};
	$scope.change = function() {
		userService.changeMyPassword($scope.form.currentPassword, $scope.form.newPassword, function() {
			$translate(["views.changePassword.success"]).then(function (msg) {
				messages.success(msg["views.changePassword.success"], {onClose: function() {
					$ionicHistory.goBack();
				}});
			});
		});
	};
})
.controller('MyProfileCtrl', function($scope, userService, $ionicHistory) {
	$scope.user = {};
	$scope.loading = false;
	$scope.doRefresh = function() {
			if (!$scope.loading) {
				$scope.loading = true;
				$scope.newMessages = 0;
				userService.getCurrentUser(function(user) {
					$scope.user = user;
					$scope.loading = false;
					$scope.$broadcast('scroll.refreshComplete');
				});
			}
	};
	$scope.$on('$stateChangeSuccess', function (event, toState, toParams, fromState, fromParams) {
    if (toState.name == "app.myProfile") {
    	$scope.doRefresh();
    }
  });
	$scope.update = function() {
	  	userService.update($scope.user, function() {
	      $ionicHistory.goBack();
	    });
	};
  $scope.photo = function() {
  	userService.photo($scope.user, function(version) {
  		$scope.user.hasImage = true;
  		$scope.user.imageVersion = version;
  	});
  };
  $scope.gallery = function() {
	userService.photoFromGallery($scope.user, function(version) {
  		$scope.user.hasImage = true;
  		$scope.user.imageVersion = version;
  	});
  };
})
.controller('WaitTransactionNumberCtrl', function($scope, bankAccountService, purchaseService, $ionicHistory, storage, $state, messages, $filter) {
	$scope.loading = false;
	$scope.remaining = {};
	$scope.remaining.hours = "--";
	$scope.remaining.minutes = "--";
	$scope.remaining.seconds = "--";

	bankAccountService.listByCompany(companyId, function(companyAccounts) {
		if(companyAccounts != null && companyAccounts.length > 0) {
			$scope.purchase = storage.getObject('pendingPurchase');
			$scope.companyAccounts = companyAccounts;
			var interval;
			purchaseService.getByUUID($scope.purchase.uuid, function(data) {
				if(data != null) {
					$scope.purchase.remainingSeconds = data.secondsRemaining;
					interval = setInterval(function(){
						$scope.$apply(function() {
							var secs = $scope.purchase.remainingSeconds = $scope.purchase.remainingSeconds - 1;
							$scope.remaining.hours = Math.floor(secs / 3600).toString().paddingLeft("00");;
							secs = secs % 3600;
							$scope.remaining.minutes = Math.floor(secs / 60).toString().paddingLeft("00");;
							$scope.remaining.seconds = (secs % 60).toString().paddingLeft("00");;
							if($scope.purchase.remainingSeconds <= 0) {
								$scope.cancelReservation();
								messages.error($filter('translate')('views.waitBankTransaction.purchaseExpired'), {onClose: function() {
									storage.putObject('pendingPurchase', null);
									$ionicHistory.nextViewOptions({
							          disableAnimate: true,
							          disableBack: true
							        });
									$state.go("app.events");
								}});
							}
						});
					}, 1000);
				} else {
					messages.error($filter('translate')('views.waitBankTransaction.purchaseExpired'), {onClose: function() {
						storage.putObject('pendingPurchase', null);
						$ionicHistory.nextViewOptions({
				          disableAnimate: true,
				          disableBack: true
				        });
						$state.go("app.events");
					}});
				}
			});

			$scope.cancelReservation = function() {
				clearInterval(interval);
				$scope.purchase.remainingSeconds = null;
				storage.putObject('pendingPurchase', null);
				purchaseService.cancel($scope.purchase.id, function() {
					storage.putObject('pendingPurchase', null);
					$ionicHistory.nextViewOptions({
			          disableAnimate: true,
			          disableBack: true
			        });
					$state.go("app.events");
				});
			};
			
			$scope.setTransactionId = function() {
				if($scope.purchase.transactionId) {
					purchaseService.setTransactionId($scope.purchase.uuid, $scope.purchase.zone.id, $scope.purchase.transactionId, function() {
						clearInterval(interval);
						$scope.purchase.remainingSeconds = null;
						storage.putObject('pendingPurchase', null);
                        setTimeout(function() {
                            $ionicHistory.nextViewOptions({
                              disableAnimate: true,
                              disableBack: true
                            });
                            $state.go('app.myTickets');
                        }, 500);
					});
				} else {
					messages.error($filter('translate')('views.waitBankTransaction.noTransactionId'));
				}
			};
		} else {
			messages.error($filter('translate')('views.waitBankTransaction.noAccountsConfigured'));
		}
		
	});
})
;
