angular.module('starter.services', [])
// underscore library
.factory('_', function($window) {
  return $window._;
})
.factory('socketIo', function($window) {
  var socketIo = {};
  socketIo.connected = false;
  socketIo.getSocket = function() {
    var io = $window.io;
    var socket = io(socketio_endpoint, {'force new connection': true});

    return socket;
  };
  return socketIo;
})
// common messages
.factory('messages', function($ionicPopup, $translate) {
  var messages = {};
  var defaultMsgs;
  $translate(['global.success.title', 'global.success.button','global.error.title', 'global.error.button', 'global.confirm.title', 'global.confirm.areYourSure', 'global.confirm.yes', 'global.confirm.no', 'global.info.title', 'global.info.button']).then(function (msgs) {
    defaultMsgs = msgs;
  });
  messages.error = function(msg, options) {
    var defaultOpts =  {title: defaultMsgs['global.error.title'], button: defaultMsgs['global.error.button'], onClose: function() {}};
    var opts = angular.extend({}, defaultOpts, options);

    $ionicPopup.show({
      template: msg,
      title: opts.title,
      buttons: [
        {
          text: opts.button,
          type: 'button-assertive',
          onTap: function(e) {
            opts.onClose();
          }
        }
      ]
    });
  };
  messages.success = function(msg, options) {
    var defaultOpts =  {title: defaultMsgs['global.success.title'], button: defaultMsgs['global.success.button'], onClose: function() {}};
    var opts = angular.extend({}, defaultOpts, options);

    $ionicPopup.show({
      template: msg,
      title: opts.title,
      buttons: [
        {
          text: opts.button,
          type: 'button-balanced',
          onTap: function(e) {
            opts.onClose();
          }
        }
      ]
    });
  };
  messages.info = function(msg, options) {
	    var defaultOpts =  {title: defaultMsgs['global.info.title'], button: defaultMsgs['global.info.button'], onClose: function() {}};
	    var opts = angular.extend({}, defaultOpts, options);

	    $ionicPopup.show({
	      template: msg,
	      title: opts.title,
	      buttons: [
	        {
	          text: opts.button,
	          type: 'button-balanced',
	          onTap: function(e) {
	            opts.onClose();
	          }
	        }
	      ]
	    });
	  };
  messages.confirm = function(options) {
    var defaultOpts =  {
      title: defaultMsgs['global.confirm.title'],
      yesText: defaultMsgs['global.confirm.yes'],
      onYes: function() {},
      noText: defaultMsgs['global.confirm.no'],
      onNo: function() {},
      msg: defaultMsgs['global.confirm.areYourSure']
    };
    var opts = angular.extend({}, defaultOpts, options);

    $ionicPopup.show({
      title: opts.title,
      template: opts.msg,
      buttons: [
        {
          text: opts.noText,
          type: 'button-stable',
          onTap: function(e) {
            opts.onNo();
          }
        },
        {
          text: opts.yesText,
          type: 'button-assertive',
          onTap: function(e) {
            opts.onYes();
          }
        }
      ]
    });
  };
  messages.validation = function(type, name) {
    $translate(name).then(function (field) {
      $translate('global.validations.' + type, {field: field}).then(function (m) {
        messages.error(m);
      });
    });
  };
  return messages;
})
// stores key / values
.factory('storage', function() {
  var storage = {};
  storage.put = function(key, value) {
    if (value == null) {
    	localStorage.removeItem(key);
    } else {
    	window.localStorage[key] = value;
    }
  };
  storage.get = function(key) {
    return window.localStorage[key];
  };
  storage.putObject = function(key, value) {
  	if (value == null) {
    	localStorage.removeItem(key);
    } else {
    	window.localStorage[key] = JSON.stringify(value);
    }
  };
  storage.getObject = function(key, defaultValue) {
    var stringValue = window.localStorage[key];
    var value = null;
    if (stringValue) {
      value = JSON.parse(stringValue);
    }
    if (!value && defaultValue) {
    	value = defaultValue;
    }
    return value;
  };
  return storage;
})
.factory('badgeService', function($cordovaBadge) {
	var badgeService = {};
	badgeService.set = function(b) {
		if (autoLogin == null) {
			if (typeof(cordova) != 'undefined') {
				ionic.Platform.ready(function() {
					$cordovaBadge.hasPermission().then(function(result) {
						$cordovaBadge.set(b);
					}, function(error) {
						$cordovaBadge.promptForPermission();
						$cordovaBadge.set(b);
			    });
				});
			}
		}
	};
	return badgeService;
})
// central backend communication
.factory('backend', function(storage, messages, $ionicLoading, $http, $translate, $state, $cordovaFileTransfer, $rootScope) {
  var backend = {};
  backend.getBaseEndpoint = function() {
//  	var v = storage.get('baseEndpoint');
//  	if (!v) {
//  		v = app_endpoint;
//  	}
//  	return v;
	  return app_endpoint;
  };
  backend.setBaseEndpoint = function(baseEndpoint) {
  	storage.put('baseEndpoint', baseEndpoint);
  	backend.baseEndpoint = baseEndpoint;
  	$rootScope.app_endpoint = baseEndpoint;
  };
  backend.baseEndpoint = backend.getBaseEndpoint();
  backend.sessionExpired = false;
  backend.defaultOptions = {blockScreen: true};
  backend.isLoading = 0;
  backend.userInfoKey = "musart-userInfo";
  backend.post = function(url, data, callback, options) {
    var opts = angular.extend({}, backend.defaultOptions, options);
    var config = {headers: {}};
    if (opts.blockScreen) {
      backend.isLoading++;
      $ionicLoading.show({template: "<ion-spinner></ion-spinner>"})
    }
    var userInfo = storage.getObject(backend.userInfoKey);
    if (!data) {
      data = {};
    }
    var headers = {};
    headers.versionNumber = versionNumber;
    if (userInfo) {
      headers.token = userInfo.token;
    }
    if (locale) {
      headers.locale = locale;
    }
    var res = $http({
        method: 'POST',
        url: backend.baseEndpoint + url,
        data: data,
        headers: headers
    });
    res.success(function(data, status, headers, config) {
        if (opts.blockScreen) {
          backend.isLoading--;
          if (backend.isLoading == 0) {
            $ionicLoading.hide();
          }
        }
        if (data.status == 'SUCCESS') {
          if (callback) callback(data);
        } else {
          if (data.code == "invalid_token") {
            if (backend.sessionExpired === false) {
                backend.sessionExpired = true;
                setTimeout(function() {
                  $state.go("login");
                  messages.error(data.msg, function() {
                    backend.sessionExpired = false;
                  });
                },500);
              }
          } else if (data.code == "noBusiness") {
            if (backend.businessId) {
              messages.error(data.msg);
            }
          } else {
        	if (opts.onError) {
        		opts.onError(data);
        	} else {
        		messages.error(data.msg);
        	}
          }
        }
    });
    res.error(function(data, status, headers, config) {
        if (opts.blockScreen) {
          backend.isLoading--;
          if (backend.isLoading == 0) {
            $ionicLoading.hide();
          }
        }
        $translate("global.error.unknown").then(function(error) {
          messages.error(error);
        });
    });
  }
  backend.upload = function(url, filePath, fileName, data, callback) {
    var userInfo = storage.getObject(backend.userInfoKey);
    if (!data) {
      data = {};
    }
    var headers = {};
    if (userInfo) {
      headers.token = userInfo.token;
    }
    if (locale) {
      headers.locale = locale;
    }
    $ionicLoading.show({
      template: '0 %'
    });
    var options = {};
    options.fileKey = "file";
    options.fileName = fileName;
    options.chunkedMode = false;
    options.mimeType = "image/jpeg";
    options.headers = headers;
    options.params = data;

    $cordovaFileTransfer.upload(backend.baseEndpoint + url, filePath, options, true)
      .then(function(result) {
        var data = JSON.parse(result.response);
        $ionicLoading.hide();
        if (data.status == 'SUCCESS') {
          if (callback) callback(data);
        } else {
          messages.error(data.msg);
        }
      }, function(err) {
        messages.error(err);
      }, function (progress) {
        var p = Math.floor((progress.loaded / progress.total) * 100);
        $ionicLoading.show({
          template: p + ' %'
        });
      });
  };
  return backend;
})
.factory('notifications', function(backend, storage, $rootScope, $ionicPlatform, $cordovaPushV5, $state, $ionicHistory, $cordovaToast, badgeService) {
  var notifications = {};
  var options = {
      android: {
        senderID: gcm_sender_id
      },
      ios: {
        alert: "true",
        badge: "true",
        sound: "true"
      },
      windows: {}
  };
  notifications.createDevice = function(token) {
    var params = {
      platform: ionic.Platform.platform(),
      token: token
    };
    backend.post('/device/create', params, function(data) {
        storage.put('notifications_device_id', data.data.id);
    }, {blockScreen: false});
  };
  notifications.bindEvent = function() {
    ionic.Platform.ready(function() {
        $rootScope.$on('$cordovaPushV5:notificationReceived', function(event, data){
            // data.message,
            // data.title,
            // data.count,
            // data.sound,
            // data.image,
            // data.additionalData
            if (data.additionalData && data.additionalData.foreground) {
                $cordovaToast.showLongTop(data.message);
            } else {
                // do something with push notifications data...
            }
        });
    });
  };
  notifications.register = function(userInfo) {
      document.addEventListener("deviceready", function(){
          var info = storage.getObject(backend.userInfoKey);
          if (info) {
              $cordovaPushV5.initialize(options).then(function() {
                  // start listening for new notifications
                  $cordovaPushV5.onNotification();
                  // start listening for errors
                  $cordovaPushV5.onError();
                  
                  // register to get registrationId
                  $cordovaPushV5.register().then(function(registrationId) {
                    notifications.createDevice(registrationId);
                  })
              });
          }
      });
  };
  notifications.unregister = function(){
    if(ionic.Platform.isAndroid() || ionic.Platform.isIOS()) {
      document.addEventListener("deviceready", function(){
        var params = {
          platform: ionic.Platform.platform(),
          token: storage.get('notifications_token')
        };
        backend.post('/device/delete', [storage.get('notifications_device_id')], function() {
        }, {blockScreen: false});
        storage.put('notifications_device_id', null);
        $cordovaPushV5.unregister().then(function(result) {
          // Success!
        }, function(err) {
          // Error
        });
      });
    }
  };
  return notifications;
})
.factory('userUtils', function(storage, backend, $rootScope, $ionicPopup, notifications, $filter, messages) {
  var userUtils = {};
  userUtils.login = function(email, password, callback) {
  	storage.put('notifications_device_id', null);
  	backend.post('/user/login?email=' + encodeURIComponent(email) + "&password=" + encodeURIComponent(password), null, function(data) {
      storage.putObject(backend.userInfoKey, data.data);
      notifications.register(data.data);
      $rootScope.$emit('app.userChanged');
      callback();
    });
  };
  userUtils.signUp = function(user, callback) {
  	storage.put('notifications_device_id', null);
  	backend.post('/user/signUp?companyId=' + companyId, user, function(data) {
      storage.putObject(backend.userInfoKey, data.data);
      notifications.register(data.data);
      $rootScope.$emit('app.userChanged');
      callback();
    });
  };
  userUtils.getUserInfo = function() {
    var info = storage.getObject(backend.userInfoKey);
    return info;
  };
  userUtils.getBadge = function() {
  	backend.post('/user/getBadge', null, function(data) {
      callback(data.data);
    }, {blockScreen: false});
  };
  userUtils.facebookLogin = function(token, callback) {
	  	storage.put('notifications_device_id', null);
	  	backend.post('/user/facebookLogin?companyId=' + companyId + '&token=' + encodeURIComponent(token), null, function(data) {
	      storage.putObject(backend.userInfoKey, data.data);
	      notifications.register(data.data);
	      $rootScope.$emit('app.userChanged');
	      callback();
	    });
	  };
  userUtils.showLogin = function(callback) {
	var userInfo = userUtils.getUserInfo();
	if (!userInfo) {
		var $scope = $rootScope.$new();
		$scope.user = {};
		$scope.state = {name: 'facebook'};
		
		var facebookLogin = function(authResponse) {
			var token = authResponse.accessToken;
			userUtils.facebookLogin(token, function(data) {
				$scope.user = {};
				$scope.loginPopup.close();
				callback && callback();
				$scope.$destroy();
			});
		}
		
		var fbLoginSuccess = function(response) {
		    if (!response.authResponse){
		        console.log(response);
		    	messages.error($filter('translate')('global.error.facebookConnect'));
		    	return;
		    }
		    facebookLogin(response.authResponse);
		};

		var fbLoginError = function(error){
		    console.log(error);
			messages.error($filter('translate')('global.error.facebookConnect'));
		};
		var locked = false;
		$scope.facebookSignIn = function() {
		    console.log(new Date(), "facebook login 1");
			facebookConnectPlugin.getLoginStatus(function(success){
			    if (locked == false) {
			        locked = true;
    			    console.log(new Date(), "facebook login 2");
    				if(success.status === 'connected'){
    					facebookLogin(success.authResponse);
    				} else {
    					// FB permissions here: https://developers.facebook.com/docs/facebook-login/permissions/v2.4
    					facebookConnectPlugin.login(['email', 'public_profile'], fbLoginSuccess, fbLoginError);
    				}
    				setTimeout(function(){ locked = false; }, 500);
			    }
			});
		};
		$scope.signIn = function() {
			userUtils.login($scope.user.email, $scope.user.password, function(data) {
				$scope.user = {};
				$scope.loginPopup.close();
				callback && callback();
				$scope.$destroy();
			});
		};
		$scope.signUp = function() {
			userUtils.signUp($scope.user, function(data) {
				$scope.user = {};
				$scope.loginPopup.close();
				callback && callback();
				$scope.$destroy();
			});
		};
		$scope.loginPopup = $ionicPopup.show({
		  cssClass: 'login-dialog',
		  templateUrl: "templates/login.html",
		  scope: $scope,
		});
		$scope.closeLoginPopup = function() {
			$scope.loginPopup.close();
			$scope.$destroy();
		};
	} else {
		callback && callback();
	}
  };
  return userUtils;
})
.factory('userService', function(backend, userUtils, storage, messages, notifications, $translate, $rootScope, camera, badgeService) {
  var userService = {};
  userService.showLogin = userUtils.showLogin;
  userService.changeMyPassword = function(currentPassword, newPassword, callback) {
    backend.post('/user/changeMyPassword?currentPassword=' + encodeURIComponent(currentPassword) + "&newPassword=" + encodeURIComponent(newPassword), null, function(data) {
      storage.putObject(backend.userInfoKey, data.data);
      callback();
    });
  };
  userService.getCurrentUser = function(callback) {
    backend.post('/user/getCurrentUser', null, function(data) {
      callback(data.data);
    }, {blockScreen: false});
  };
  userService.isLoggedIn = function() {
    return (storage.getObject(backend.userInfoKey) != null);
  };
  userService.getUserInfo = userUtils.getUserInfo;
  userService.photo = function(user, callback) {
  	camera.getSquaredPicture(function(data) {
  		var params = {x: Math.floor(data.x), y: Math.floor(data.y), width: Math.floor(data.width), height: Math.floor(data.height)};
  		backend.upload('/upload/userProfilePicture/' + user.id, data.filePath, 'user.jpg', params, function(data) {
  			var userInfo = userService.getUserInfo();
  			var imageVersion = 0;
  			if (user.imageVersion) {
  				imageVersion = user.imageVersion + 1;
  			}
  			userInfo.imageVersion = imageVersion;
  			userInfo.hasImage = true;
  			storage.putObject(backend.userInfoKey, userInfo);
  			$rootScope.$emit('app.userChanged');
  			if (callback) callback(imageVersion);
  		});
  	});
  };
  userService.photoFromGallery = function(user, callback) {
  	camera.getSquaredPictureFromGallery(function(data) {
  		var params = {x: Math.floor(data.x), y: Math.floor(data.y), width: Math.floor(data.width), height: Math.floor(data.height)};
  		backend.upload('/upload/userProfilePicture/' + user.id, data.filePath, 'user.jpg', params, function(data) {
  			var userInfo = userService.getUserInfo();
  			var imageVersion = 0;
  			if (user.imageVersion) {
  				imageVersion = user.imageVersion + 1;
  			}
  			userInfo.imageVersion = imageVersion;
  			userInfo.hasImage = true;
  			storage.putObject(backend.userInfoKey, userInfo);
  			$rootScope.$emit('app.userChanged');
  			if (callback) callback(imageVersion);
  		});
  	});
  };
  userService.logout = function(callback) {
    notifications.unregister();
    storage.put(backend.userInfoKey, null);
    storage.put("tickets", null);
    $rootScope.tickets = null;
    //badgeService.set(0);
    $rootScope.$emit('app.userChanged');
    callback();
  };
  userService.update = function(user, callback) {
	console.log(user);
    backend.post('/user/update', user, function(data) {
		var userInfo = userService.getUserInfo();
		userInfo.name = user.name;
		storage.putObject(backend.userInfoKey, userInfo);
		$rootScope.$emit('app.userChanged');
		callback();
    });
  };
  return userService;
})
.factory('contactUsService', function(backend, $ionicModal) {
  var contactUsService = {};
  contactUsService.send = function(form, callback) {
    /*var url = '/fb/mobile/support';
    backend.post(url, form, function(data) {
      callback();
    });*/
    // TODO: implement
    callback();
  };
  contactUsService.showModal = function($scope) {
    $ionicModal.fromTemplateUrl('templates/contactUs.html', {
        scope: $scope,
        animation: 'slide-in-up',
        focusFirstInput: true
    }).then(function(modal) {
        $scope.modal = modal;
        $scope.modal.show();
    });
    $scope.cancel = function() {
      $scope.modal.hide();
    };
    $scope.$on('$destroy', function() {
      $scope.modal.remove();
    });
    $scope.send = function() {
      contactUsService.send($scope.form, function() {
        $scope.modal.hide();
      });
    };
  };
  return contactUsService;
})
.factory('camera', function($cordovaCamera, messages, $translate, $jrCrop, $cordovaFile, $ionicLoading) {
  var camera = {};
  camera.getPicture = function(callback) {
  	$ionicLoading.show();
    var options = {
      quality: 75,
      destinationType: Camera.DestinationType.FILE_URI,
      sourceType: Camera.PictureSourceType.CAMERA,
      allowEdit: false,
      encodingType: Camera.EncodingType.JPEG,
      targetWidth: 1024,
      targetHeight: 1024,
      saveToPhotoAlbum: false,
      correctOrientation: true
    };
    ionic.Platform.ready(function() {
	    $cordovaCamera.getPicture(options).then(function(image) {
	    	$ionicLoading.hide();
	      if (callback) callback(image);
	    }, function(err) {
	    	$ionicLoading.hide();
	      //messages.error(err);
	    });
    });
  };
  camera.getPictureFromGallery = function(callback) {
  	$ionicLoading.show();
    var options = {
      quality: 75,
      destinationType: Camera.DestinationType.FILE_URI,
      sourceType: Camera.PictureSourceType.PHOTOLIBRARY,
      allowEdit: false,
      encodingType: Camera.EncodingType.JPEG,
      targetWidth: 1024,
      targetHeight: 1024,
      saveToPhotoAlbum: false,
      correctOrientation: true
    };
    ionic.Platform.ready(function() {
	    $cordovaCamera.getPicture(options).then(function(image) {
	    	$ionicLoading.hide();
	      if (callback) callback(image);
	    }, function(err) {
	    	$ionicLoading.hide();
	      //messages.error(err);
	    });
    });
  };
  camera.getSquaredPicture = function(callback){
  	camera.getPicture(function(imagePath) {
      var currentFolder = imagePath.substring(0, imagePath.lastIndexOf("/"));
      var currentFile = imagePath.substring(imagePath.lastIndexOf("/") + 1, imagePath.length);
      var fileName = "lastPhoto.jpg";
      $cordovaFile.moveFile(currentFolder, currentFile, cordova.file.dataDirectory, fileName).then(function(success){
        $translate(['imageCropper.choose', 'imageCropper.cancel']).then(function (msgs) {
          var filePath = cordova.file.dataDirectory + fileName;
          $jrCrop.crop({
              url: filePath + "?_=" + Math.random(),
              width: 300,
              height: 300,
              chooseText: msgs['imageCropper.choose'],
              cancelText: msgs['imageCropper.cancel']
          }).then(function(e) {
          	if (callback) {
          		callback({filePath: filePath, x: e.x, y: e.y, width: e.width, height: e.height});
          	}
          }, function() {
              // User canceled or couldn't load image.
          });
        });
        }, function(error){
          messages.error(error);
        });
    }, function(err) {
      messages.error(err);
    });
  };
  camera.getSquaredPictureFromGallery = function(callback){
  	camera.getPictureFromGallery(function(imagePath) {
  	  var currentFolder = imagePath.substring(0, imagePath.lastIndexOf("/"));
      var currentFile = imagePath.substring(imagePath.lastIndexOf("/") + 1, imagePath.length);
      currentFile = currentFile.substring(0, currentFile.indexOf("?"));
      var fileName = "lastPhoto.jpg";
      $cordovaFile.copyFile(currentFolder, currentFile, cordova.file.dataDirectory, fileName).then(function(success){
        $translate(['imageCropper.choose', 'imageCropper.cancel']).then(function (msgs) {
          var filePath = cordova.file.dataDirectory + fileName;
          $jrCrop.crop({
              url: filePath + "?_=" + Math.random(),
              width: 300,
              height: 300,
              chooseText: msgs['imageCropper.choose'],
              cancelText: msgs['imageCropper.cancel']
          }).then(function(e) {
          	if (callback) {
          		callback({filePath: filePath, x: e.x, y: e.y, width: e.width, height: e.height});
          	}
          }, function() {
              // User canceled or couldn't load image.
          });
        });
        }, function(error){
          messages.error(error);
        });
    }, function(err) {
      messages.error(err);
    });
  };
  return camera;
})
.factory('imageCache', function(backend, _, $cordovaFileTransfer, $cordovaFile) {
	var imageCache = {};
	var cacheFolder = null;
	if (typeof cordova != "undefined") {
		ionic.Platform.ready(function() {
			cacheFolder = cordova.file.dataDirectory;
			$cordovaFile.createDir(cacheFolder, "events")
		});
	}
	imageCache.event = function(event, eventId, companyId) {
		if (cacheFolder) {
			var fileName = cacheFolder + "/events/" + eventId + ".jpg";
			$cordovaFile.checkFile(cacheFolder, "events/" + eventId + ".jpg").then(function() {
				event.img = fileName;
			}, function() {
				// file is missing
				var url = backend.getBaseEndpoint() + '/download/eventImage/' + companyId + '/' + eventId;
				event.img = "img/default-event.png";
				$cordovaFileTransfer.download(url, fileName).then(function() {
					event.img = fileName;
				});
			});
		} else {
			// browser, no cache
			event.img = backend.getBaseEndpoint() + '/download/eventImage/' + companyId + '/' + eventId;
		}
	};
	return imageCache;
})
.factory('eventService', function(backend, _, $ionicModal, $rootScope, userUtils, storage, messages, $filter, purchaseService, bankAccountService, $state, $ionicHistory, imageCache) {
  var eventService = {};
  function completeData(event) {
	  event.currency = {symbol: '₡'};
	  event.date = new Date(event.date);
	  event.day = event.date.getDate();
	  event.month = event.date.getMonth() + 1;
	  imageCache.event(event, event.id, event.company.id);
	  var price = null;
	  _.each(event.zones, function(it) {
		  if (!price) {
			  price = event.isPresale ? it.preSale : it.regularPrice
		  } else {
			  price = Math.min(event.isPresale ? it.preSale : it.regularPrice, price);
		  }
	  });
	  event.price = price;
  }
  eventService.list = function(offset, filter, state, callback) {
    if (state == 'all') {
		backend.post('/event/list?max=10&offset=' + offset + "&filter=" + encodeURIComponent(filter), null, function(data) {
	    	if (data.data) {
	    		_.each(data.data, function(event) {
	    			completeData(event);
	    		});
	    	}
	    	callback(data.data);
	    }, {blockScreen: false});
    } else if (state == 'liked') {
    	backend.post('/event/listLiked?max=10&offset=' + offset + "&filter=" + encodeURIComponent(filter), null, function(data) {
	    	if (data.data) {
	    		_.each(data.data, function(event) {
	    			completeData(event);
	    		});
	    	}
	    	callback(data.data);
	    }, {blockScreen: false});
    }
  };
  eventService.listByCompany = function(companyId, offset, filter, state, callback) {
	  if (state == 'liked') {
		  backend.post('/event/listLiked?companyId=' + companyId + '&max=10&offset=' + offset + "&filter=" + encodeURIComponent(filter), null, function(data) {
			  if (data.data) {
				  _.each(data.data, function(event) {
					  completeData(event);
				  });
			  }
			  callback(data.data);
		  }, {blockScreen: false});
	  } else {
		  backend.post('/event/listByCompany?companyId=' + companyId + '&max=10&offset=' + offset + "&filter=" + encodeURIComponent(filter), null, function(data) {
			  if (data.data) {
				  _.each(data.data, function(event) {
					  completeData(event);
				  });
			  }
			  callback(data.data);
		  }, {blockScreen: false});  
	  }
  };
  eventService.openEvent = function(event, $scope) {
	  $scope.event = event;
		$scope.buyTicket = function(zone) {
			userUtils.showLogin(function() {
				purchaseService.reserve(zone.id, 2, function(purchase) {
					$scope.zoneSelected = zone;
					$scope.purchase = purchase;
					$scope.buy = {quantity: "" + purchase.tickets.length, promo: null, promoCode: ""};
					$scope.applyPromo = function() {
						purchaseService.applyPromo($scope.buy.promoCode, zone.id, $scope.purchase.id, function(promo) {
							if (promo) {
								if (promo.isAvailable) {
									$scope.buy.promo = promo;
								} else {
									messages.error($filter('translate')('views.buyTicket.unavailablePromoCode'));
								}
							} else {
								messages.error($filter('translate')('views.buyTicket.invalidPromoCode'));
							}
						});
					};
					$scope.removePromo = function() {
						purchaseService.removePromo($scope.purchase.id, function() {
							$scope.buy.promo = null;
						});
					};
					$scope.showTerms = function() {
						$scope.modalUrl = $filter("translate")("views.buyTicket.termsUrl");
						$scope.modalTitle = $filter("translate")("views.buyTicket.termsTitle");
						$ionicModal.fromTemplateUrl('templates/externalPage.html', {
					      scope: $scope,
					      animation: 'slide-in-up'
					    }).then(function(modal) {
					    	$scope.externalPageModal = modal;
						    $scope.externalPageModal.show();
					    });
						$scope.$on('$destroy', function() {
							$scope.externalPageModal.remove();
						});
					};
					$scope.getTotal = function() {
						var total = $scope.buy.quantity * (event.isPresale ? zone.preSale : zone.regularPrice);
						if ($scope.buy.promo) {
							total = total * (1 - ($scope.buy.promo.percentage / 100));
						}
						return total;
					};
					$scope.cancelPurchase = function() {
						unwatchQuantity();
						purchaseService.cancel($scope.purchase.id, function() {
							$scope.buyTicketModal.hide();
						});
					};
					$scope.confirmPurchase = function() {
						unwatchQuantity();
						
						if($scope.getTotal() > 0) {
							purchaseService.waitForPayment($scope.purchase.uuid, function() {
								$scope.purchase.payedAmount = $scope.event.currency.symbol + $scope.getTotal();
								$scope.purchase.zone = zone;
								$scope.purchase.remainingSeconds = 21600;
								storage.putObject('pendingPurchase', $scope.purchase);
								setTimeout(function() {
	                                $ionicHistory.nextViewOptions({
	                                  disableAnimate: true,
	                                  disableBack: true
	                                });
	                                $state.go('waitTransactionNumber');
	                            }, 500);

							});
						} else {
							console.log($scope.purchase);
							purchaseService.validateFreePurchase($scope.purchase.uuid, zone.id, function() {
								storage.putObject('pendingPurchase', null);
								setTimeout(function() {
	                                $ionicHistory.nextViewOptions({
	                                  disableAnimate: true,
	                                  disableBack: true
	                                });
	                                $state.go('app.myTickets');
	                            }, 500);
							});
						}   
					};
					var unwatchQuantity = $scope.$watch('buy.quantity', function(newValue) {
						if ($scope.purchase && $scope.purchase.tickets.length != newValue) {
				        	purchaseService.changeTickets($scope.purchase.id, zone.id, newValue, function(tickets) {
				        		$scope.purchase.tickets = tickets;
							}, function(data) {
								$scope.buy.quantity = "" + $scope.purchase.tickets.length;
								messages.error(data.msg);
							});
				        }
				    });
					$ionicModal.fromTemplateUrl('templates/buyTicket.html', {
				      scope: $scope,
				      animation: 'slide-in-up'
				    }).then(function(modal) {
				    	$scope.buyTicketModal = modal;
					    $scope.buyTicketModal.show();
				    });
					$scope.$on('$destroy', function() {
						$scope.buyTicketModal.remove();
					});
					$scope.$on('$stateChangeStart', function(event, toState, toParams, fromState, fromParams){
						$scope.buyTicketModal.hide();
                        $scope.eventModal.hide();
					});
				});
			});
		};
		$scope.toggleLike = function(event) {
			userUtils.showLogin(function() {
				eventService.toggleLike(event, function(liked){
					event.liked = liked;
				});
			});
		};
		$ionicModal.fromTemplateUrl('templates/event.html', {
	      scope: $scope,
	      animation: 'slide-in-up'
	    }).then(function(modal) {
	    	$scope.eventModal = modal;
		    $scope.eventModal.show();
	    });
		$scope.$on('$destroy', function() {
			$scope.eventModal.remove();
		});
  };
  eventService.toggleLike = function(event, callback) {
	  backend.post('/event/toggleLike?eventId=' + event.id, null, function(data) {
		callback(data.data);
	  });
  };
  eventService.view = function(event, callback) {
	  if (userUtils.getUserInfo()) {
		  backend.post('/event/view?eventId=' + event.id, null, function(data) {
			callback && callback(data.data);
		  });
	  }
  };
  eventService.countNotViewed = function(callback) {
	  if (userUtils.getUserInfo()) {
		  backend.post('/event/countNotViewed', null, function(data) {
			callback(data.data);
		  });
	  } else {
		  callback(0);
	  }
  };
  return eventService;
})
.factory('purchaseService', function(backend, $filter, _, storage, $rootScope, imageCache) {
  var purchaseService = {};
  purchaseService.getByUUID = function(uuid, callback) {
    backend.post('/purchase/getByUUID?uuid=' + uuid, null, function(data) {
      callback(data.data);
    });
  };
  purchaseService.reserve = function(eventZoneId, max, callback) {
    backend.post('/purchase/reserve?eventZoneId=' + eventZoneId + '&max=' + max, null, function(data) {
      callback(data.data);
    });
  };
  purchaseService.cancel = function(purchaseId, callback) {
    backend.post('/purchase/cancel?purchaseId=' + purchaseId, null, function(data) {
      callback(data.data);
    });
  };
  function completeTicket(ticket) {
	  imageCache.event(ticket, ticket.eventId, ticket.companyId);
  }
  var sortFunction = function(a, b) {
	  return new Date(a.date).getTime() - new Date(b.date).getTime();
  };
  purchaseService.setTransactionId = function(uuid, eventZoneId, transactionId, callback) {
    backend.post('/purchase/setTransactionId?uuid=' + uuid + "&eventZoneId=" + eventZoneId + "&transactionId=" + transactionId, null, function(data) {
    	var tickets = storage.getObject('tickets', []);
    	var ticket = data.data;
    	completeTicket(ticket);
    	tickets.push(ticket);
    	storage.putObject('tickets', tickets);
    	$rootScope.tickets = tickets;
    	$rootScope.tickets.sort(sortFunction);
    	callback(data.data);
    });
  };
  purchaseService.waitForPayment = function(uuid, callback) {
    backend.post('/purchase/waitForPayment?uuid=' + uuid, null, function() {
    	callback();
    });
  };
  purchaseService.confirm = function(purchaseId, eventZoneId, transactionId, callback) {
    backend.post('/purchase/confirm?uuid=' + purchaseId + "&eventZoneId=" + eventZoneId + "&transactionId=" + transactionId, null, function(data) {
    	var tickets = storage.getObject('tickets', []);
    	var ticket = data.data;
    	completeTicket(ticket);
    	tickets.push(ticket);
    	storage.putObject('tickets', tickets);
    	$rootScope.tickets = tickets;
    	$rootScope.tickets.sort(sortFunction);
    	callback(data.data);
    });
  };
  purchaseService.validateFreePurchase = function(purchaseId, eventZoneId, callback) {
    backend.post('/purchase/validateFreePurchase?uuid=' + purchaseId + "&eventZoneId=" + eventZoneId, null, function(data) {
    	var tickets = storage.getObject('tickets', []);
    	var ticket = data.data;
    	completeTicket(ticket);
    	tickets.push(ticket);
    	storage.putObject('tickets', tickets);
    	$rootScope.tickets = tickets;
    	$rootScope.tickets.sort(sortFunction);
    	callback(data.data);
    });
  };
  purchaseService.syncTickets = function() {
	  var tickets = storage.getObject('tickets', []);
	  $rootScope.tickets = [];
	  backend.post('/purchase/listMyTickets', null, function(data) {
		  if(data.status == "SUCCESS") {
			  _.each(data.data, function(serverTicket) {
				  var existing = _.findWhere(tickets, {uuid: serverTicket.uuid});
				  if (!existing) {
					  completeTicket(serverTicket);
					  $rootScope.tickets.push(serverTicket);
				  } else {
					  imageCache.event(existing, existing.eventId, existing.companyId);
					  if(existing.status < 6) {
						  existing.status = serverTicket.status;
					  }
					  $rootScope.tickets.push(existing);
				  }
			  });
			  $rootScope.tickets.sort(sortFunction);
			  storage.putObject('tickets', $rootScope.tickets);
		  }
		  
	  }, {blockScreen: false});
  };
  purchaseService.applyPromo = function(code, eventZoneId, purchaseId, callback) {
    backend.post('/purchase/applyPromo?code=' + code + '&eventZoneId=' + eventZoneId + '&purchaseId=' + purchaseId, null, function(data) {
      callback(data.data);
    });
  };
  purchaseService.removePromo = function(purchaseId, callback) {
    backend.post('/purchase/removePromo?purchaseId=' + purchaseId, null, function(data) {
      callback();
    });
  };
  purchaseService.changeTickets = function(purchaseId, eventZoneId, number, callback, onError) {
    backend.post('/purchase/changeTickets?purchaseId=' + purchaseId + "&eventZoneId=" + eventZoneId + "&number=" + number, null, function(data) {
      callback(data.data);
    }, {onError: onError});
  };
  return purchaseService;
})
.factory('bankAccountService', function(backend, _, $ionicModal, $rootScope, messages) {
  var bankAccountService = {};

  bankAccountService.listByCompany = function(companyId, callback) {
	  backend.post('/bankAccount/listByCompany?companyId=' + companyId, null, function(data) {
		  callback(data.data);
	  }, {blockScreen: false});
  };
  return bankAccountService;
})
.factory('userReadMessageService', function(backend, $filter, _) {
  var userReadMessageService = {};
  userReadMessageService.create = function(userReadMessage, callback) {
    backend.post('/userReadMessage/create', userReadMessage, function(data) {
      callback(data.data);
    }, {blockScreen: false});
  };
  return userReadMessageService;
})
.factory('messageService', function(backend, $ionicModal, $ionicScrollDelegate, socketIo, $filter, userReadMessageService, _, userService, messageTemplateService) {
  var messageService = {};
  messageService.listByStudent = function(studentId, first, callback){
  	var url = '/message/listByStudent?max=15&studentId=' + studentId;
  	if (first) {
  		url += "&first=" + first;
  	}
  	backend.post(url, null, function(data) {
      callback(data.data);
    }, {blockScreen: false});
  };
  messageService.countByStudent = function(studentId, callback){
  	backend.post('/message/countByStudent?studentId=' + studentId, null, function(data) {
      callback(data.data);
    }, {blockScreen: false});
  };
  messageService.create = function(message, callback) {
    backend.post('/message/create', message, function(data) {
      callback(data.data);
    }, {blockScreen: false});
  };
  function initConversation(scope, student, callback) {
  	scope.student = student;
  	scope.userInfo = userService.getUserInfo();
  	scope.conversationModalTitle = student.name + " " + student.lastName;
  	scope.conversationMessages = [];
  	scope.conversationTotalMessages = -1;
  	scope.loadingMessages = false;
  	scope.cancelSelectMessageTemplate = function() {
  		scope.selectMessageTemplateModal.hide();
  	};
  	scope.showMessageTemplates = function() {
  		scope.doSelectMessageTemplate = function(template) {
  			var callbacks = [];
  			var out = template.body;
  			var variables = config.messageTemplateVariables();
  	  	_.each(variables, function(it) {
  	  		if (it.type == 'expression') {
	  	  		var value = scope.$eval(it.expression);
	  	  		out = out.replace(new RegExp(it.value, "g"), value);
  	  		} else if (it.type == 'day_of_week') {
  	  			if (out.indexOf(it.value) != -1) {
	  	  			callbacks.push(function() {
	  	  				$ionicModal.fromTemplateUrl('templates/dayOfWeekPicker.html', {
	  	  		      scope: scope,
	  	  		      animation: 'slide-in-up',
	  	  		      focusFirstInput: true
	  	  			  }).then(function(modal) {
	  	  			      scope.dayOfWeekModal = modal;
	  	  			      scope.daysOfWeek = [];
	  	  			      for (var i = 1; i <= 7; i++) {
	  	  			      	scope.daysOfWeek.push({number: i, name: $filter('translate')('dayOfWeek.' + i)});
										}
	  	  			      scope.cancelDayOfWeekPicker = function() {
	  	  			      	scope.dayOfWeekModal.hide();
	  	  			      };
	  	  			      scope.selectDayOfWeek = function(day) {
	  	  			      	out = out.replace(new RegExp(it.value, "g"), day.name);
	  	  			      	scope.dayOfWeekModal.hide();
	  	  			      	callbacks.shift();
	  	  			      	callbacks[0]();
	  	  			      }
	  	  				    scope.dayOfWeekModal.show();
	  	  			  });	
	  	  			});
	  				}
  	  		} else if (it.type == 'time') {
  	  			if (out.indexOf(it.value) != -1) {
	  	  			callbacks.push(function() {
	  	  				$ionicModal.fromTemplateUrl('templates/timePicker.html', {
	  	  		      scope: scope,
	  	  		      animation: 'slide-in-up',
	  	  		      focusFirstInput: true
	  	  			  }).then(function(modal) {
	  	  			      scope.timePickerModal = modal;
	  	  			      scope.daysOfWeek = [];
	  	  			      var d = new Date();
	  	  			      var pad = function(num) {
		  	  			        var s = num + "";
		  	  			        while (s.length < 2) s = "0" + s;
		  	  			        return s;
		  	  			    }
	  	  			      scope.timePicker = {
	  	  			      		hours: d.getHours() % 12,
	  	  			      		minutes: pad(d.getMinutes()),
	  	  			      		meridian: d.getHours() < 12 ? "AM" : "PM",
	  	  			      		changeMeridian: function() {
	  	  			      			this.meridian = scope.timePicker == "AM" ? "PM" : "AM";
	  	  			      		},
	  	  			      		increaseHours: function() {
	  	  			      			this.hours++;
	  	  			      			if (this.hours == 13) {
	  	  			      				this.hours = 1;
	  	  			      			}
	  	  			      		},
	  	  			      		decreaseHours: function() {
	  	  			      			this.hours--;
	  	  			      			if (this.hours == 0) {
	  	  			      				this.hours = 12;
	  	  			      			}
	  	  			      		},
	  	  			      		increaseMinutes: function() {
	  	  			      			this.minutes = parseInt(this.minutes);
	  	  			      			this.minutes++;
	  	  			      			if (this.minutes == 60) {
	  	  			      				this.minutes = 0;
	  	  			      			}
	  	  			      			this.minutes = pad(this.minutes);
	  	  			      		},
	  	  			      		decreaseMinutes: function() {
	  	  			      			this.minutes = parseInt(this.minutes);
	  	  			      			this.minutes--;
	  	  			      			if (this.minutes == -1) {
	  	  			      				this.minutes = 59;
	  	  			      			}
	  	  			      			this.minutes = pad(this.minutes);
	  	  			      		}
	  	  			      };
	  	  			      if (scope.timePicker.hours == 0) {
	  	  			      	scope.timePicker.hours = 12;
	  	  			      }
	  	  			      scope.cancelTimePicker = function() {
	  	  			      	scope.timePickerModal.hide();
	  	  			      };
	  	  			      scope.selectTime = function() {
	  	  			      	var t = scope.timePicker.hours + ":" + scope.timePicker.minutes + " " + scope.timePicker.meridian;
	  	  			      	out = out.replace(new RegExp(it.value, "g"), t);
	  	  			      	scope.timePickerModal.hide().then(function() {
	  	  			      		callbacks.shift();
		  	  			      	callbacks[0]();
	  	  			      	});
	  	  			      }
	  	  				    scope.timePickerModal.show();
	  	  			  });	
	  	  			});
	  				}
  	  		} else if (it.type == 'number') {
  	  			if (out.indexOf(it.value) != -1) {
	  	  			callbacks.push(function() {
	  	  				scope.selectedNumber = {value: 0};
	  	  				$ionicModal.fromTemplateUrl('templates/numberPicker.html', {
	  	  		      scope: scope,
	  	  		      animation: 'slide-in-up',
	  	  		      focusFirstInput: true
	  	  			  }).then(function(modal) {
	  	  			  	  scope.numberPickerModal = modal;
	  	  			      scope.cancelNumberPicker = function() {
	  	  			      	scope.numberPickerModal.hide();
	  	  			      };
	  	  			      scope.selectNumber = function() {
	  	  			      	out = out.replace(new RegExp(it.value, "g"), scope.selectedNumber.value);
	  	  			      	scope.numberPickerModal.hide().then(function() {
	  	  			      		callbacks.shift();
		  	  			      	callbacks[0]();
	  	  			      	});
	  	  			      }
	  	  				    scope.numberPickerModal.show();
	  	  			  });	
	  	  			});
	  				}
  	  		}
  	  	});
  	  	callbacks.push(function() {
  	  		scope.message.text = out;
    	  	scope.selectMessageTemplateModal.hide();
  	  	});
  	  	console.log(callbacks);
  	  	callbacks[0]();
  		};
  		scope.loadingMessageTemplates = false;
  		scope.lastMessageTemplateLoaded = false;
  		scope.messageTemplates = [];
  		scope.doRefreshMessageTemplates = function() {
  	    scope.messageTemplates = [];
  	    scope.lastMessageTemplateLoaded = false;
  	    scope.loadMoreMessageTemplates();
  	  };
  	  scope.loadMoreMessageTemplates = function() {
  	    if (!scope.loadingMessageTemplates) {
  	      scope.loadingMessageTemplates = true;
  	      messageTemplateService.list(scope.messageTemplates.length, function(messageTemplates) {
  	        if (!messageTemplates || messageTemplates.length == 0) {
  	          scope.lastMessageTemplateLoaded = true;
  	        } else {
  	          scope.messageTemplates = scope.messageTemplates.concat(messageTemplates);
  	        }
  	        scope.$broadcast('scroll.infiniteScrollComplete');
  	        scope.$broadcast('scroll.refreshComplete');
  	        scope.loadingMessageTemplates = false;
  	      });
  	    }
  	  };
  		if (!scope.selectMessageTemplateModal) {
  	  	$ionicModal.fromTemplateUrl('templates/selectMessageTemplate.html', {
  	      scope: scope,
  	      animation: 'slide-in-up'
  		  }).then(function(modal) {
  		      scope.selectMessageTemplateModal = modal;
  			    scope.selectMessageTemplateModal.show();
  		  });
  		  scope.$on('$destroy', function() {
  		    scope.selectMessageTemplateModal.remove();
  		  });
    	} else {
    		scope.selectMessageTemplateModal.show();
    	}
  	};
  	scope.loadMore = function() {
  		scope.loadingMessages = true;
  		var first = null;
  		if (scope.conversationMessages && scope.conversationMessages.length > 0) {
  			first = scope.conversationMessages[0].id;
  		}
  		messageService.listByStudent(student.id, first, function(messages) {
  			if (messages) {
	  			var scroll = scope.conversationMessages.length == 0;
	    		scope.conversationMessages = messages.reverse().concat(scope.conversationMessages);
	    		if (scroll) {
	    			$ionicScrollDelegate.scrollBottom(true);
	    		}
  			}
    		scope.loadingMessages = false;
    	});
  	};
  	scope.loadMore();
  	messageService.countByStudent(student.id, function(c) {
  		scope.conversationTotalMessages = c;
  	});
  	scope.message = {text: '', student: student};
  	var socket = socketIo.getSocket();
  	var socket_channel = "musart_student_" + student.id;
  	socket.emit('join comments', socket_channel);
  	scope.createMessage = function() {
  		if (scope.message.text != '') {
  			var userInfo = userService.getUserInfo();
  			var msg = angular.extend({}, scope.message);
	  		var msgEntry = {text: msg.text, read: true,  user: userInfo};
	  		scope.conversationMessages.push(msgEntry);
	  		$ionicScrollDelegate.scrollBottom(true);
	  		scope.message.text = '';
	  		messageService.create(msg, function(messageCreated) {
	  			scope.conversationMessages = _.filter(scope.conversationMessages, function(it) {
	  				return it.dateCreated;
	  			});
	  			scope.conversationMessages.push(messageCreated);
	  			scope.conversationTotalMessages++;
	  			scope.$broadcast('new-message');
	  			$ionicScrollDelegate.scrollBottom(true);
	  			socket.emit('chat message', JSON.stringify({id: socket_channel, msg: JSON.stringify(messageCreated)}));
	  		});
  		}
  	};
  	scope.chatMessageDate = function(input){
  		var output = "...";
  		if (input) {
	  		var d = new Date(input);
	  		var today = $filter('date')(new Date(), date_format);
	  		output = $filter('date')(d, date_time_format);
	  		if (output.indexOf(today) == 0) {
	  			output = $filter('date')(d, time_format);
	  		}
  		}
  		return output;
  	};
  	var reading = [];
  	scope.markRead = function(message) {
  		if (!message.read && reading.indexOf(message.id) == -1) {
  			reading.push(message.id);
	  		var read = {message: message};
	  		userReadMessageService.create(read, function() {
	  			message.read = true;
	  			reading = _.without(reading, message.id);
	  		});
  		}
  	};
    socket.on('chat message', function(msg){
    	scope.$apply(function() {
    		scope.conversationMessages.push(JSON.parse(msg));
        scope.conversationTotalMessages++;
        $ionicScrollDelegate.scrollBottom(true);
    	});
    });
    scope.closeConversationModal = function() {
	  	socket.disconnect();
	    scope.conversationModal.hide();
	    if (callback) callback();
	  };
  }
  messageService.showConversation = function(scope, student, callback) {
  	if (!scope.conversationModal) {
	  	$ionicModal.fromTemplateUrl('templates/conversation.html', {
	      scope: scope,
	      animation: 'slide-in-up',
	      focusFirstInput: true
		  }).then(function(modal) {
		      scope.conversationModal = modal;
		      initConversation(scope, student, callback);
			    scope.conversationModal.show();
		  });
		  scope.$on('$destroy', function() {
		    scope.conversationModal.remove();
		  });
  	} else {
  		initConversation(scope, student, callback);
  		scope.conversationModal.show();
  	}
  };
  return messageService;
})
.factory('notificationService', function(backend, $filter, _, badgeService, $rootScope, userService) {
  var notificationService = {};
  notificationService.list = function(offset, callback) {
      backend.post('/notification/list?max=10&offset=' + offset, null, function(data) {
          callback(data.data);
      }, {blockScreen: false});
  };
  notificationService.view = function(notification, callback) {
      backend.post('/notification/view?notificationId=' + notification.id, null, function(data) {
        callback && callback(data.data);
      });
  };
  notificationService.countNotViewed = function(callback) {
      backend.post('/notification/countNotViewed', null, function(data) {
        callback(data.data);
      });
  };
  notificationService.updateBadge = function() {
      var userInfo = userService.getUserInfo();
      if (userInfo) {
          notificationService.countNotViewed(function(n) {
              badgeService.set(n);
              $rootScope.notViewedNotifications = n;
          });
      }
  };
  return notificationService;
})
;